-----------------------------------------------------------------------------------------------
-- Client Lua Script for ReCap
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- ReCap Module Definition
-----------------------------------------------------------------------------------------------
local ReCap = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function ReCap:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here

    return o
end

function ReCap:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 

-----------------------------------------------------------------------------------------------
-- ReCap OnLoad
-----------------------------------------------------------------------------------------------
function ReCap:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("ReCap.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- ReCap OnDocLoaded
-----------------------------------------------------------------------------------------------
function ReCap:OnDocLoaded()
	--Load sprites
	Apollo.LoadSprites("spr_recap.xml");
	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "frm_main", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)

		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("recap", "OnReCapOn", self)
		Apollo.RegisterEventHandler("CombatLogDamage", "OnCombatLogDamage", self)
		Apollo.RegisterEventHandler("UnitEnteredCombat", "OnEnteredCombat", self)


		-- Do additional Addon initialization here
		if not self.abilityTable then
			self.abilityTable = {}
		end
		
		self.playerList = {}
	end
end

-----------------------------------------------------------------------------------------------
-- ReCap Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here
--Save variables
function ReCap:OnSave(eLevel)
  if eLevel ~= GameLib.CodeEnumAddonSaveLevel.Character then
    return nil
  end

  local tSavedData = { }
  
  tSavedData = {
    abilityTable = self.abilityTable;
  }

  return tSavedData
end

--When we load in
function ReCap:OnRestore(eType, savedData)
		if savedData.abilityTable then
			self.abilityTable = savedData.abilityTable;
		end
end

-- on SlashCommand "/recap"
function ReCap:OnReCapOn()
	self.wndMain:Invoke() -- show the window
end

--When a player takes or deals damage
function ReCap:OnCombatLogDamage(tEventArgs)
--unitCaster, splCallingSpell, unitTarget
	local unitTarget = tEventArgs.unitTarget;
	local spellName = tEventArgs.splCallingSpell:GetName();
	local targetKilled  = tEventArgs.bTargetKilled;
	local spellIcon = tEventArgs.splCallingSpell:GetIcon();

	if unitTarget:IsACharacter() and self:isWatched(spellName) then
		local failItem = Apollo.LoadForm(self.xmlDoc, "frm_player", self.wndMain:FindChild("frm_listContainer"), self);
		
		--Set text
		failItem:FindChild("lbl_player"):SetText(unitTarget:GetName() .. " got hit by " .. spellName .. ".");
		
		--Set icon, if not found then question mark
		if spellIcon ~= "" then
			failItem:FindChild("frm_spellIcon"):SetSprite(tEventArgs.splCallingSpell:GetIcon());
		else
			failItem:FindChild("frm_spellIcon"):SetSprite("IconSprites:Icon_SkillMedic_suture");
			failItem:FindChild("frm_spellIcon"):Enable(false);
		end
		
		--If the target died
		if targetKilled then
			failItem:SetBGColor("red");
		end
		
		--Insert into our fail table
		table.insert(self.playerList, failItem);
		
		--Redraw the list
		self:redrawList();
	end
end

--Finds an ability in self.abilityTable and returns true or false
function ReCap:isWatched(spellName)
	for i,v in ipairs(self.abilityTable) do
		if string.lower(self.abilityTable[i]) == string.lower(spellName) then
			return true
		end
	end
	
	return false
end

--Basically goes through the list and sets their positions
function ReCap:redrawList()
	for i,v in ipairs(self.playerList) do
		local wndHeight = self.playerList[i]:GetHeight();
		local spacing = 10;
		local vOffset = wndHeight*(i-1) + spacing+spacing*(i-1);
		
		self.playerList[i]:SetAnchorOffsets(10,vOffset,-10,vOffset+wndHeight);
	end
end

function ReCap:OnEnteredCombat(unit, inCombat)
	if unit == GameLib.GetPlayerUnit() and inCombat then
		for i,v in ipairs(self.playerList) do
			self.playerList[i]:Close();
		end
		
		self.playerList = {}
	end
end

---------------------------------------------------------------------------------------------------
-- frm_main Functions
---------------------------------------------------------------------------------------------------

function ReCap:onBtnClose(wndControl)
	local window = wndControl:GetParent();
	local btnHandle = self.wndMain:FindChild("btn_addSpellDialogue");
	btnHandle:Enable(true);
	
	--Lazy way to close input form, lel
	if self.input then
		self.input:Close();
	end
	
	window:Close();
end

function ReCap:onBtnAddSpellDialogue()
	local btnHandle = self.wndMain:FindChild("btn_addSpellDialogue");
	btnHandle:Enable(false);
	
	self.input = Apollo.LoadForm(self.xmlDoc, "frm_input", nil, self);
end

---------------------------------------------------------------------------------------------------
-- frm_input Functions
---------------------------------------------------------------------------------------------------

function ReCap:onBtnAddSpell(wndControl)
	local input = self.input:FindChild("in_spellName"):GetText();
	
	if input ~= "Spell Name" and input then
		table.insert(self.abilityTable, tostring(input));
		Print(input .. " - Spell Added.");
	else
		Print("Please add a real spell :)");
	end
end

function ReCap:onBtnResetFilter()
	self.abilityTable = {}
	Print("Spell filter cleared!");
end

---------------------------------------------------------------------------------------------------
-- frm_player Functions
---------------------------------------------------------------------------------------------------

function ReCap:onBtnPrintInChat(wndControl)
	local window = wndControl:GetParent();
	local failString = "/say " .. window:FindChild("lbl_player"):GetText();
	wndControl:Enable(false);
	
	ChatSystemLib.Command(failString);
end

-----------------------------------------------------------------------------------------------
-- ReCap Instance
-----------------------------------------------------------------------------------------------
local ReCapInst = ReCap:new()
ReCapInst:Init()
